# Copyright 2020-2021, Hewlett Packard Enterprise Development LP
'''
A set of routines for creating or reading from an existing timestamp file.
Created on April 27, 2020

@author: jsl
'''
import logging
from datetime import datetime, timedelta

from liveness import TIMESTAMP_PATH


LOGGER = logging.getLogger(__name__)


class Timestamp(object):
    def __init__(self, path=TIMESTAMP_PATH, when=None):
        '''
        Creates a new timestamp representation to <path>; on initialization,
        this timestamp is written to disk in a persistent fashion.

        Newly initialized timestamps with a path reference to an existing file
        overwrites the file in question.
        '''
        self.path = path
        with open(self.path, 'w') as timestamp_file:
            if not when:
                # how?
                timestamp_file.write(str(datetime.now().timestamp()))
            else:
                timestamp_file.write(str(when.timestamp()))

    def __str__(self):
        return 'Timestamp from %s; age: %s' % (self.value.strftime("%m/%d/%Y, %H:%M:%S"), self.age)

    @classmethod
    def byref(cls, path):
        """
        Creates a new instance of a Timestamp without initializing it to disk.
        This is useful if you simply want to check the existence of a timestamp
        without altering it.
        """
        self = super().__new__(cls)
        self.path = path
        return self

    @property
    def value(self):
        """
        The timestamp value, as stored on disk. This property does not cache
        the value; instead it reads it each time the property is accessed.
        """
        try:
            with open(self.path, 'r') as timestamp_file:
                return datetime.fromtimestamp(float(timestamp_file.read().strip()))
        except FileNotFoundError:
            LOGGER.warning("Timestamp never intialized to '%s'" % (self.path))
            return datetime.fromtimestamp(0)
        except ValueError:
            # CASMCMS-6856: There is an edgecase where backgrounded writes of a timestamp
            # occur between a new file descriptor being opened and written to; in this
            # scenario, the new file is created but has not been written to. As a result,
            # conversion to a float creates a ValueError. When this happens, we know that
            # the timestamp is currently being written, so simply returning the current
            # timestamp is acceptable.
            return datetime.fromtimestamp(0)

    @property
    def age(self):
        """
        How old this timestamp is, implemented as a timedelta object.
        """
        return datetime.now() - self.value

    @property
    def max_age(self):
        """
        The maximum amount of time that can elapse before we consider the timestamp
        as invalid.

        This property is intended to be overwritten by subclasses.
        """
        computation_time = timedelta(seconds=30)
        return computation_time

    @property
    def alive(self):
        """
        Returns a true or false, depending on if this service is considered alive/viable.
        True if the service has a new enough timestamp; false otherwise.
        """
        return self.age < self.max_age



# Copyright 2020 Hewlett Packard Enterprise Development LP
import os

WORKING_DIRECTORY = '/var/'
TIMESTAMP_PATH = os.path.join(WORKING_DIRECTORY, 'timestamp')

os.makedirs(WORKING_DIRECTORY, exist_ok=True)



#!/usr/bin/env python
# Copyright 2020 Hewlett Packard Enterprise Development LP
'''
This entrypoint is used to determine if this service is still active/alive
from a kubernetes liveness probe perspective.

This service is deemed to be 'alive' and healthy if the
main loop has executed relatively recently. The period of time for how frequently
the agent checks for operational work is defined as a function of event frequency from
kubernetes, so this liveness probe needs a larger than normal window to account for
periods of time without a recent liveness cycle.

This main routine exists as an example; each project should implement their own
__main__ routine within their own liveness project. Then, a helm chart deployment may
reference the projects newly created module by adding:

      livenessProbe:
        exec:
         command:
         - python3
         - "-m"
         - "myservice.liveness"
        initialDelaySeconds: 10
        periodSeconds: 30

to the associated helmchart.

Created on April 30, 2020

@author: jsl
'''

import sys
import logging
import os

from ..liveness import TIMESTAMP_PATH
from .timestamp import Timestamp


LOGGER = logging.getLogger('liveness.main')
DEFAULT_LOG_LEVEL = logging.INFO


def setup_logging():
    log_format = "%(asctime)-15s - %(levelname)-7s - %(name)s - %(message)s"
    requested_log_level = os.environ.get('LOG_LEVEL', DEFAULT_LOG_LEVEL)
    log_level = logging.getLevelName(requested_log_level)
    logging.basicConfig(level=log_level, format=log_format)


if __name__ == '__main__':
    setup_logging()
    timestamp = Timestamp.byref(TIMESTAMP_PATH)
    if timestamp.alive:
        LOGGER.info("%s is considered valid; the application is alive!", timestamp)
        sys.exit(0)
    else:
        LOGGER.warning("Timestamp is no longer considered valid.")
        sys.exit(1)



# setup.py for ims-python-helper
# Copyright 2020 Hewlett Packard Enterprise Development LP
import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

with open('.version', 'r') as fh:
    version = fh.read().strip()

setuptools.setup(
    name="liveness",
    version=version,
    package_dir = {'liveness': 'src/liveness'},
    packages=['liveness'],
    install_requires=[
      'liveness',
    ],
    author="Hewlett Packard Enterprise Development LP",
    author_email="cray-oss@hpe.com",
    description="Library for simplifying liveness checks for python based infrastructure",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/Cray-HPE/k8s-liveness",
    download_url="https://github.com/Cray-HPE/k8s-liveness/archive/liveness-1.1.2.tar.gz",
    keywords="Cray/HPE Kubernetes Liveness Helm",
    classifiers=(
          'Development Status :: 5 - Production/Stable',
          'Intended Audience :: Developers',
          'Topic :: Software Development :: Build Tools',
          'License :: OSI Approved :: MIT License',   
          'Programming Language :: Python :: 3.6'
    ),
)
